/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * (C) Copyright 2007
 * Stelian Pop <stelian@popies.net>
 * Lead Tech Design <www.leadtechdesign.com>
 */
#ifndef __ASM_ARM_DMA_MAPPING_H
#define __ASM_ARM_DMA_MAPPING_H

#include <asm/cache.h>
#include <cpu_func.h>
#include <linux/dma-direction.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <malloc.h>

static inline void *dma_alloc_coherent(size_t len, unsigned long *handle)
{
#if IS_ENABLED(CONFIG_SYS_ARM_DMA_ALLOC_COHERENT)
	*handle = (unsigned long)memalign(MMU_SECTION_SIZE, ALIGN(len, MMU_SECTION_SIZE));
	if (!*handle)
		return NULL;

	mmu_set_region_dcache_behaviour((phys_addr_t)*handle,
					ALIGN(len, MMU_SECTION_SIZE),
					DCACHE_OFF);
#else
	*handle = (unsigned long)memalign(ARCH_DMA_MINALIGN, ROUND(len, ARCH_DMA_MINALIGN));
#endif

	return (void *)*handle;
}

static inline void dma_free_coherent(void *addr)
{
	free(addr);
}

#endif /* __ASM_ARM_DMA_MAPPING_H */
